<?php
/**
 * List the subpages for a node type
 */
function node_subpages_admin_list($type) {
  $type_url_str = str_replace('_', '-', $type->type);
  
  
  $subpages = node_subpages_list_for_type($type->type);
  $rows = array();
  
  foreach ($subpages as $path => $details) {
    if ($details['default_view_page']) {
      $tab_type = 'Default tab';
    }
    elseif ($details['field']) {
      $tab_type = 'Field: ' . $details['field'];
    }
    elseif ($details['view']) {
      $tab_type = 'View: ' . $details['view'];
    }
    else {
      $tab_type = 'Unknown';
    }
    
    $operations = array();
    if ($details['id'] > 0) {
      $operations[] = l('edit', 'admin/content/node-type/'. $type_url_str .'/subpages/edit/' . $details['id']);
      $operations[] = l('delete', 'admin/content/node-type/'. $type_url_str .'/subpages/delete/' . $details['id']);
    }
    $rows[] = array(
      $path,
      $details['title'],
      $tab_type,
      implode(' | ', $operations),
    );
  }
  
  $headers = array(
    t('Sub-path'),
    t('Title'),
    t('Type'),
    t('Operations'),
  );


  if (empty($rows)) {
    $rows[] = array(
      array(
        'data' => t('No subpages have been added for this content type.'),
        'colspan' => count($headers),
      )
    );
  }

  
  
  $output = theme('table', $headers, $rows);
  $output .= l('Add another subpage', 'admin/content/node-type/'. $type_url_str .'/subpages/add');
  return $output;
}


/**
 * Form to add a new subpage to the given content type
 */
function node_subpages_add_subpage(&$form_state, $type, $id = NULL) {
  $form = array();
  $defaults = array();
  
  // If editing an existing subpage, load the details for use as default
  // values in the form
  if ($id > 0) {
    $form['#id'] = $id;
    $defaults = node_subpages_load_specific($id);
  }

  
  $form['#node_type'] = $type;
  
  $form['subpath'] = array(
    '#type' => 'textfield',
    '#title' => t('Subpath'),
    '#description' => t('Subpath that will be used in the URL for this subpage on the node.'),
    '#size' => 40,
    '#maxlength' => 255,
    '#required' => TRUE,
    '#default_value' => $defaults['subpath'],
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Tab Title'),
    '#description' => t('Displayed as the text of the tab link for this subpage.'),
    '#size' => 40,
    '#maxlength' => 255,
    '#required' => TRUE,
    '#default_value' => $defaults['title'],
  );
  
  $form['data_source'] = array(
    '#type' => 'fieldset',
    '#title' => t('Data Source'),
    '#description' => t('Select the source for the content of this subpage. Only choose one.'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
  );
  
  
  if (module_exists('content')) {
    $cck_info = content_types($type->type);
    if ($cck_info['fields']) {
      $options = array('' => '-- Select One --');
      foreach ($cck_info['fields'] as $field_key => $field_details) {
        $options[$field_key] = $field_details['widget']['label'] . ' (' . $field_key . ')';
      }
    
      $form['data_source']['field'] = array(
        '#type' => 'select',
        '#title' => t('CCK Field'),
        '#multiple' => FALSE,
        '#description' => t('CCK field to use as the content of this subpage.'),
        '#options' => $options,
        '#default_value' => $defaults['field'],
      );
    }
  }
  
  if (module_exists('views')) {
    $views = views_get_all_views();
    $options = array('' => '-- Select One --');
    foreach ($views as $view_name => $view) {
      foreach ($view->display as $display_name => $display) {
        $options[$view_name . ':' . $display_name] = $view_name . ': ' . $display->display_title;
      }
    }
    
    $form['data_source']['view'] = array(
      '#type' => 'select',
      '#title' => t('View'),
      '#multiple' => FALSE,
      '#description' => t('View to display as content of the subpage.'),
      '#options' => $options,
      '#default_value' => $defaults['view'] . ':' . $defaults['view_display'],
    );
  }
  
  if ($id > 0) {
    $form['update'] = array(
      '#type' => 'submit',
      '#value' => t('Update'),
    );
  }
  else {
    $form['add'] = array(
      '#type' => 'submit',
      '#value' => t('Add'),
    );
  }
  return $form;
  
  
}


/**
 * Submit the subpage add form, to create a new subpage type
 */
function node_subpages_add_subpage_submit($form, &$form_state) {
  $values = $form_state['values'];
  $node_type = $form['#node_type'];
  
  $record = array(
    'node_type' => $node_type->type,
    'subpath' => $values['subpath'],
    'title' => $values['title'],
  );
  
  if ($values['field']) {
    $record['source_type'] = 'cck';
    $record['source_data'] = $values['field'];
  }
  elseif ($values['view']) {
    $record['source_type'] = 'view';
    $record['source_data'] = $values['view'];
  }
  
  if ($form['#id'] > 0) {
    $record['id'] = $form['#id'];
    $saved = drupal_write_record('node_subpages', $record, 'id');
  }
  else {
    $saved = drupal_write_record('node_subpages', $record);
  }
  
  if($saved) {
    if ($saved == SAVED_NEW) {
      drupal_set_message(t('New subpage saved.'));
    }
    else {
      drupal_set_message(t('Subpage updated.'));
    }
    
    $type_url_str = str_replace('_', '-', $node_type->type);
    
    // Rebuild the menu cache so that the path is updated
    menu_rebuild();
    
    drupal_goto('admin/content/node-type/'. $type_url_str .'/subpages');
  }
  else {
    drupal_set_message(t('Error saving subpage. Please try again.'));
    return FALSE;
  }
}


/**
 * Form to delete subpage: confirm before deleting
 */
function node_subpages_delete_subpage(&$form_state, $node_type, $id) {
  $type_url_str = str_replace('_', '-', $node_type->type);
  $cancel_path = 'admin/content/node-type/'. $type_url_str .'/subpages';
  $details = node_subpages_load_specific($id);
  $question = t('Are you sure you want to delete the %subpage subpage from %content_type?', array(
    '%subpage' => $details['title'],
    '%content_type' => $node_type->name,
  ));

  $form = array();
  $form['#id'] = $id;
  $form['#node_type'] = $node_type;
  return confirm_form($form, $question, $cancel_path, NULL, t('Delete'));
}

/**
 * Submit the subpage deletion confirmation form
 */
function node_subpages_delete_subpage_submit($form, &$form_state) {
  db_query('DELETE FROM {node_subpages} WHERE id=%d', $form['#id']);
  drupal_set_message(t('Subpage deleted.'));
  
  $type_url_str = str_replace('_', '-', $form['#node_type']->type);
  $redirect_path = 'admin/content/node-type/'. $type_url_str .'/subpages';
  drupal_goto($redirect_path);
}